@@ -147,31 +147,32 @@ void odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *m
 void odbc_errs_add_rdbms(struct _sql_errors *errs, TDS_UINT native, const char *sqlstate, const char *msg, int linenum,
 			 int msgstate, const char *server, int row);
 
+/** Descriptor header */
 struct _dheader
 {
 	SQLSMALLINT sql_desc_alloc_type;
-	SQLINTEGER sql_desc_bind_type;
-	SQLULEN sql_desc_array_size;
 	/* TODO SQLLEN ?? see http://support.microsoft.com/default.aspx?scid=kb;en-us;298678 */
 	SQLSMALLINT sql_desc_count;
+	SQLINTEGER sql_desc_bind_type;
+	SQLULEN sql_desc_array_size;
 	SQLUSMALLINT *sql_desc_array_status_ptr;
 	SQLULEN *sql_desc_rows_processed_ptr;
 	SQLLEN *sql_desc_bind_offset_ptr;
 };
 
+/** Descriptor record */
 struct _drecord
 {
-	SQLUINTEGER sql_desc_auto_unique_value;
 	DSTR sql_desc_base_column_name;
 	DSTR sql_desc_base_table_name;
+	SQLUINTEGER sql_desc_auto_unique_value;
 	SQLINTEGER sql_desc_case_sensitive;
 	DSTR sql_desc_catalog_name;
-	SQLSMALLINT sql_desc_concise_type;
 	SQLPOINTER sql_desc_data_ptr;
+	SQLSMALLINT sql_desc_concise_type;
 	SQLSMALLINT sql_desc_datetime_interval_code;
 	SQLINTEGER sql_desc_datetime_interval_precision;
 	SQLLEN sql_desc_display_size;
-	SQLSMALLINT sql_desc_fixed_prec_scale;
 	SQLLEN *sql_desc_indicator_ptr;
 	DSTR sql_desc_label;
 	SQLULEN sql_desc_length;
@@ -181,6 +182,7 @@ struct _drecord
 	const char *sql_desc_literal_suffix;
 	DSTR sql_desc_local_type_name;
 	DSTR sql_desc_name;
+	SQLSMALLINT sql_desc_fixed_prec_scale;
 	SQLSMALLINT sql_desc_nullable;
 	SQLINTEGER sql_desc_num_prec_radix;
 	SQLLEN sql_desc_octet_length;
@@ -190,8 +192,8 @@ struct _drecord
 	SQLSMALLINT sql_desc_rowver;
 	SQLSMALLINT sql_desc_scale;
 	DSTR sql_desc_schema_name;
-	SQLSMALLINT sql_desc_searchable;
 	DSTR sql_desc_table_name;
+	SQLSMALLINT sql_desc_searchable;
 	SQLSMALLINT sql_desc_type;
 	/* this point to a constant buffer, do not free or modify */
 	const char *sql_desc_type_name;
@@ -206,6 +208,13 @@ struct _hdesc
 	struct _sql_errors errs;	/* do not reorder this field */
 	tds_mutex mtx;
 	int type;
+	/**
+	 * Nested descriptor to use.
+	 * -1: nested, cannot have focus.
+	 * 0 : self.
+	 * >0: descriptor from IPD.
+	 */
+	int focus;
 	SQLHANDLE parent;
 	struct _dheader header;
 	struct _drecord *records;
@@ -357,6 +366,7 @@ struct _hsattr
 	DSTR qn_msgtext;
 	DSTR qn_options;
 	SQLUINTEGER qn_timeout;
+	SQLUINTEGER param_focus;
 };
 
 typedef enum
@@ -514,7 +524,8 @@ bool get_login_info(HWND hwndParent, TDSLOGIN * login);
 	ODBC_PARAM(REALM) \
 	ODBC_PARAM(ServerSPN) \
 	ODBC_PARAM(AttachDbFilename) \
-	ODBC_PARAM(ApplicationIntent)
+	ODBC_PARAM(ApplicationIntent) \
+	ODBC_PARAM(Timeout)
 
 #define ODBC_PARAM(p) ODBC_PARAM_##p,
 enum {
@@ -537,10 +548,10 @@ typedef struct {
  * Parses a connection string for SQLDriverConnect().
  * \param connect_string      point to connection string
  * \param connect_string_end  point to end of connection string
- * \param connection          structure where to store information
- * \return 0 if error, 1 otherwise
+ * \param connection          structure where to store informations
+ * \return true if error, false otherwise
  */
-int odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end, TDSLOGIN * login, TDS_PARSED_PARAM *parsed_params);
+bool odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char *connect_string_end, TDSLOGIN * login, TDS_PARSED_PARAM *parsed_params);
 int odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login);
 #ifdef _WIN32
 int odbc_build_connect_string(TDS_ERRS *errs, TDS_PARSED_PARAM *params, char **out);
@@ -557,16 +568,25 @@ SQLLEN odbc_tds2sql_int4(TDS_STMT * stmt, TDS_INT *src, int desttype, TDS_CHAR *
 /*
  * descriptor.c
  */
-TDS_DESC *desc_alloc(SQLHANDLE parent, int desc_type, int alloc_type);
+typedef struct {
+	DSTR type_name;
+	TDS_DESC *apd;
+	TDS_DESC *ipd;
+} SQLTVP;
+
+TDS_DESC *desc_alloc(SQLHANDLE parent, int desc_type, SQLSMALLINT alloc_type);
 SQLRETURN desc_free(TDS_DESC * desc);
 SQLRETURN desc_alloc_records(TDS_DESC * desc, unsigned count);
 SQLRETURN desc_copy(TDS_DESC * dest, TDS_DESC * src);
 SQLRETURN desc_free_records(TDS_DESC * desc);
 TDS_DBC *desc_get_dbc(TDS_DESC *desc);
+SQLTVP *tvp_alloc(TDS_STMT *stmt);
+void tvp_free(SQLTVP *tvp);
 
 /*
  * odbc.c
  */
+
 SQLRETURN _SQLRowCount(SQLHSTMT hstmt, SQLLEN FAR * pcrow);
 
 /*
