@@ -57,7 +57,7 @@
 
 #include <freetds/tds.h>
 #include <freetds/utils.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 #include <sybfront.h>
 #include <sybdb.h>
 #include "freebcp.h"
@@ -255,7 +255,7 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 			pdata->Eflag++;
 			break;
 		case 'd':
-			tdsdump_open(NULL);
+			tdsdump_open("stderr");
 			break;
 		case 't':
 			pdata->tflag++;
