@@ -74,20 +74,23 @@ blk_alloc(CS_CONNECTION * connection, CS_INT version, CS_BLKDESC ** blk_pointer)
 
 
 CS_RETCODE
-blk_bind(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt, CS_VOID * buffer, CS_INT * datalen, CS_SMALLINT * indicator)
+blk_bind(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt_arg, CS_VOID * buffer, CS_INT * datalen, CS_SMALLINT * indicator)
 {
 	TDSCOLUMN *colinfo;
 	CS_CONNECTION *con;
 	CS_INT bind_count;
+	const CS_DATAFMT_COMMON * datafmt;
 	int i;
 
-	tdsdump_log(TDS_DBG_FUNC, "blk_bind(%p, %d, %p, %p, %p, %p)\n", blkdesc, item, datafmt, buffer, datalen, indicator);
+	tdsdump_log(TDS_DBG_FUNC, "blk_bind(%p, %d, %p, %p, %p, %p)\n", blkdesc, item, datafmt_arg, buffer, datalen, indicator);
 
-	if (!blkdesc) {
+	if (!blkdesc)
 		return CS_FAIL;
-	}
+
 	con = CONN(blkdesc);
 
+	datafmt = _ct_datafmt_common(con->ctx, datafmt_arg);
+
 	if (item == CS_UNUSED) {
 		/* clear all bindings */
 		if (datafmt == NULL && buffer == NULL && datalen == NULL && indicator == NULL ) { 
@@ -189,11 +192,19 @@ blk_default(CS_BLKDESC * blkdesc, CS_INT colnum, CS_VOID * buffer, CS_INT buflen
 }
 
 CS_RETCODE
-blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt)
+blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt_arg)
 {
 	TDSCOLUMN *curcol;
+	CS_INT status, datatype;
+	CS_DATAFMT_LARGE *datafmt;
+	CS_DATAFMT_LARGE datafmt_buf;
 
-	tdsdump_log(TDS_DBG_FUNC, "blk_describe(%p, %d, %p)\n", blkdesc, item, datafmt);
+	tdsdump_log(TDS_DBG_FUNC, "blk_describe(%p, %d, %p)\n", blkdesc, item, datafmt_arg);
+
+	if (!blkdesc)
+		return CS_FAIL;
+
+	datafmt = _ct_datafmt_conv_prepare(CONN(blkdesc)->ctx, datafmt_arg, &datafmt_buf);
 
 	if (item < 1 || item > blkdesc->bcpinfo.bindinfo->num_cols) {
 		_ctclient_msg(CONN(blkdesc), "blk_describe", 2, 5, 1, 141, "%s, %d", "colnum", item);
@@ -205,10 +216,11 @@ blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt)
 	strlcpy(datafmt->name, tds_dstr_cstr(&curcol->column_name), sizeof(datafmt->name));
 	datafmt->namelen = strlen(datafmt->name);
 	/* need to turn the SYBxxx into a CS_xxx_TYPE */
-	datafmt->datatype = _ct_get_client_type(curcol, true);
-	if (datafmt->datatype == CS_ILLEGAL_TYPE)
+	datatype = _ct_get_client_type(curcol, true);
+	if (datatype == CS_ILLEGAL_TYPE)
 		return CS_FAIL;
-	tdsdump_log(TDS_DBG_INFO1, "blk_describe() datafmt->datatype = %d server type %d\n", datafmt->datatype,
+	datafmt->datatype = datatype;
+	tdsdump_log(TDS_DBG_INFO1, "blk_describe() datafmt->datatype = %d server type %d\n", datatype,
 			curcol->column_type);
 	/* FIXME is ok this value for numeric/decimal? */
 	datafmt->maxlength = curcol->column_size;
@@ -220,15 +232,17 @@ blk_describe(CS_BLKDESC * blkdesc, CS_INT item, CS_DATAFMT * datafmt)
 	 * There are other options that can be returned, but these are the
 	 * only two being noted via the TDS layer.
 	 */
-	datafmt->status = 0;
+	status = 0;
 	if (curcol->column_nullable)
-		datafmt->status |= CS_CANBENULL;
+		status |= CS_CANBENULL;
 	if (curcol->column_identity)
-		datafmt->status |= CS_IDENTITY;
+		status |= CS_IDENTITY;
+	datafmt->status = status;
 
 	datafmt->count = 1;
 	datafmt->locale = NULL;
 
+	_ct_datafmt_conv_back(datafmt_arg, datafmt);
 	return CS_SUCCEED;
 }
 
@@ -624,7 +638,6 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 	CS_INT      *datalen = NULL;
 	CS_BLKDESC *blkdesc = (CS_BLKDESC *) bulk;
 	CS_CONTEXT *ctx = CONN(blkdesc)->ctx;
-	CS_DATAFMT srcfmt, destfmt;
 
 	tdsdump_log(TDS_DBG_FUNC, "_blk_get_col_data(%p, %p, %d)\n", bulk, bindcol, offset);
 
@@ -677,7 +690,7 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 			default:
 				printf("error not fixed length type (%d) and datalen not specified\n",
 					bindcol->column_bindtype);
-				return CS_FAIL;
+				return TDS_FAIL;
 			}
 
 		} else {
@@ -691,26 +704,28 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 
 	if (!null_column) {
 		CONV_RESULT convert_buffer;
+		CS_DATAFMT_COMMON srcfmt, destfmt;
+		CS_INT desttype;
 
 		srcfmt.datatype = srctype;
 		srcfmt.maxlength = srclen;
 
-		destfmt.datatype = _cs_convert_not_client(ctx, bindcol, &convert_buffer, &src);
-		if (destfmt.datatype == CS_ILLEGAL_TYPE)
-			destfmt.datatype  = _ct_get_client_type(bindcol, false);
-		if (destfmt.datatype == CS_ILLEGAL_TYPE)
-			return CS_FAIL;
+		desttype = _cs_convert_not_client(ctx, bindcol, &convert_buffer, &src);
+		if (desttype == CS_ILLEGAL_TYPE)
+			desttype = _ct_get_client_type(bindcol, false);
+		if (desttype == CS_ILLEGAL_TYPE)
+			return TDS_FAIL;
+		destfmt.datatype  = desttype;
 		destfmt.maxlength = bindcol->on_server.column_size;
 		destfmt.precision = bindcol->column_prec;
 		destfmt.scale     = bindcol->column_scale;
-
-		destfmt.format	= CS_FMT_UNUSED;
+		destfmt.format    = CS_FMT_UNUSED;
 
 		/* if convert return FAIL mark error but process other columns */
-		if ((result = cs_convert(ctx, &srcfmt, (CS_VOID *) src, 
+		if ((result = _cs_convert(ctx, &srcfmt, (CS_VOID *) src,
 					 &destfmt, (CS_VOID *) bindcol->bcp_column_data->data, &destlen)) != CS_SUCCEED) {
-			tdsdump_log(TDS_DBG_INFO1, "convert failed for %d \n", srcfmt.datatype);
-			return CS_FAIL;
+			tdsdump_log(TDS_DBG_INFO1, "convert failed for %d \n", srctype);
+			return TDS_FAIL;
 		}
 	}
 
