@@ -1,9 +1,6 @@
 
-#ifndef COMMON_h
-#define COMMON_h
-
-static char rcsid_common_h[] = "$Id: common.h,v 1.11 2008-07-06 16:44:24 jklowden Exp $";
-static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
+#ifndef _freetds_ctlib_common_h
+#define _freetds_ctlib_common_h
 
 extern char SERVER[512];
 extern char DATABASE[512];
