@@ -1,14 +1,9 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <ctpublic.h>
 #include "common.h"
 
-static char software_version[] = "$Id: connect_fail.c,v 1.4 2003/12/10 14:36:08 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 int
 main(int argc, char **argv)
 {
@@ -64,6 +59,6 @@ main(int argc, char **argv)
 		return ret;
 	}
 
-	fprintf(stdout, "Test succeeded\n");
+	printf("Test succeeded\n");
 	return 0;
 }
