@@ -44,6 +44,7 @@
 #include <freetds/tds.h>
 #include <freetds/bytes.h>
 #include <freetds/iconv.h>
+#include <freetds/bool.h>
 #include <freetds/utils/bjoern-utf8.h>
 
 #include "iconv_charsets.h"
@@ -135,44 +136,36 @@ put_utf8(unsigned char *buf, size_t buf_len, ICONV_CHAR c)
 static int
 get_ucs4le(const unsigned char *p, size_t len, ICONV_CHAR *out)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) p) & 3) == 0));
-
 	if (len < 4)
 		return -EINVAL;
-	*out = TDS_GET_A4LE(p);
+	*out = TDS_GET_UA4LE(p);
 	return 4;
 }
 
 static int
 put_ucs4le(unsigned char *buf, size_t buf_len, ICONV_CHAR c)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) buf) & 3) == 0));
-
 	if (buf_len < 4)
 		return -E2BIG;
-	TDS_PUT_A4LE(buf, c);
+	TDS_PUT_UA4LE(buf, c);
 	return 4;
 }
 
 static int
 get_ucs4be(const unsigned char *p, size_t len, ICONV_CHAR *out)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) p) & 3) == 0));
-
 	if (len < 4)
 		return -EINVAL;
-	*out = TDS_GET_A4BE(p);
+	*out = TDS_GET_UA4BE(p);
 	return 4;
 }
 
 static int
 put_ucs4be(unsigned char *buf, size_t buf_len, ICONV_CHAR c)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) buf) & 3) == 0));
-
 	if (buf_len < 4)
 		return -E2BIG;
-	TDS_PUT_A4BE(buf, c);
+	TDS_PUT_UA4BE(buf, c);
 	return 4;
 }
 
@@ -181,15 +174,13 @@ get_utf16le(const unsigned char *p, size_t len, ICONV_CHAR *out)
 {
 	ICONV_CHAR c, c2;
 
-	TDS_EXTRA_CHECK(assert((((uintptr_t) p) & 1) == 0));
-
 	if (len < 2)
 		return -EINVAL;
-	c = TDS_GET_A2LE(p);
+	c = TDS_GET_UA2LE(p);
 	if ((c & 0xfc00) == 0xd800) {
 		if (len < 4)
 			return -EINVAL;
-		c2 = TDS_GET_A2LE(p+2);
+		c2 = TDS_GET_UA2LE(p+2);
 		if ((c2 & 0xfc00) == 0xdc00) {
 			*out = (c << 10) + c2 - ((0xd800 << 10) + 0xdc00 - 0x10000);
 			return 4;
@@ -202,20 +193,18 @@ get_utf16le(const unsigned char *p, size_t len, ICONV_CHAR *out)
 static int
 put_utf16le(unsigned char *buf, size_t buf_len, ICONV_CHAR c)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) buf) & 1) == 0));
-
 	if (c < 0x10000u) {
 		if (buf_len < 2)
 			return -E2BIG;
-		TDS_PUT_A2LE(buf, c);
+		TDS_PUT_UA2LE(buf, c);
 		return 2;
 	}
 	if (TDS_UNLIKELY(c >= 0x110000u))
 		return -EILSEQ;
 	if (buf_len < 4)
 		return -E2BIG;
-	TDS_PUT_A2LE(buf,   0xd7c0 + (c >> 10));
-	TDS_PUT_A2LE(buf+2, 0xdc00 + (c & 0x3ffu));
+	TDS_PUT_UA2LE(buf,   0xd7c0 + (c >> 10));
+	TDS_PUT_UA2LE(buf+2, 0xdc00 + (c & 0x3ffu));
 	return 4;
 }
 
@@ -224,15 +213,13 @@ get_utf16be(const unsigned char *p, size_t len, ICONV_CHAR *out)
 {
 	ICONV_CHAR c, c2;
 
-	TDS_EXTRA_CHECK(assert((((uintptr_t) p) & 1) == 0));
-
 	if (len < 2)
 		return -EINVAL;
-	c = TDS_GET_A2BE(p);
+	c = TDS_GET_UA2BE(p);
 	if ((c & 0xfc00) == 0xd800) {
 		if (len < 4)
 			return -EINVAL;
-		c2 = TDS_GET_A2BE(p+2);
+		c2 = TDS_GET_UA2BE(p+2);
 		if ((c2 & 0xfc00) == 0xdc00) {
 			*out = (c << 10) + c2 - ((0xd800 << 10) + 0xdc00 - 0x10000);
 			return 4;
@@ -245,20 +232,18 @@ get_utf16be(const unsigned char *p, size_t len, ICONV_CHAR *out)
 static int
 put_utf16be(unsigned char *buf, size_t buf_len, ICONV_CHAR c)
 {
-	TDS_EXTRA_CHECK(assert((((uintptr_t) buf) & 1) == 0));
-
 	if (c < 0x10000u) {
 		if (buf_len < 2)
 			return -E2BIG;
-		TDS_PUT_A2BE(buf, c);
+		TDS_PUT_UA2BE(buf, c);
 		return 2;
 	}
 	if (TDS_UNLIKELY(c >= 0x110000u))
 		return -EILSEQ;
 	if (buf_len < 4)
 		return -E2BIG;
-	TDS_PUT_A2BE(buf,   0xd7c0 + (c >> 10));
-	TDS_PUT_A2BE(buf+2, 0xdc00 + (c & 0x3ffu));
+	TDS_PUT_UA2BE(buf,   0xd7c0 + (c >> 10));
+	TDS_PUT_UA2BE(buf+2, 0xdc00 + (c & 0x3ffu));
 	return 4;
 }
 
@@ -364,10 +349,10 @@ tds_sys_iconv_open (const char* tocode, const char* fromcode)
 	const char *enc_name;
 	unsigned char encodings[2];
 
-	static char first_time = 1;
+	static bool first_time = true;
 
 	if (TDS_UNLIKELY(first_time)) {
-		first_time = 0;
+		first_time = false;
 		tdsdump_log(TDS_DBG_INFO1, "Using trivial iconv\n");
 	}
 
@@ -408,7 +393,7 @@ tds_sys_iconv_open (const char* tocode, const char* fromcode)
 		fromto = Like_to_Like;
 	}
 
-	return (iconv_t) (intptr_t) fromto;
+	return (iconv_t) (TDS_INTPTR) fromto;
 } 
 
 int 
@@ -426,7 +411,7 @@ tds_sys_iconv (iconv_t cd, const char* * inbuf, size_t *inbytesleft, char* * out
 	int local_errno;
 
 #undef CD
-#define CD ((int) (intptr_t) cd)
+#define CD ((int) (TDS_INTPTR) cd)
 
 	/* iconv defines valid semantics for NULL inputs, but we don't support them. */
 	if (!inbuf || !*inbuf || !inbytesleft || !outbuf || !*outbuf || !outbytesleft)
