@@ -76,14 +76,14 @@
 #include <freetds/utils.h>
 #include <freetds/replacements.h>
 
-static int tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
-static int tds_config_env_tdsdump(TDSLOGIN * login);
+static bool tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
+static bool tds_config_env_tdsdump(TDSLOGIN * login);
 static void tds_config_env_tdsver(TDSLOGIN * login);
 static void tds_config_env_tdsport(TDSLOGIN * login);
-static int tds_config_env_tdshost(TDSLOGIN * login);
+static bool tds_config_env_tdshost(TDSLOGIN * login);
 static bool tds_read_conf_sections(FILE * in, const char *server, TDSLOGIN * login);
-static int tds_read_interfaces(const char *server, TDSLOGIN * login);
-static int parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server);
+static bool tds_read_interfaces(const char *server, TDSLOGIN * login);
+static bool parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server);
 static int tds_lookup_port(const char *portname);
 static void tds_config_encryption(const char * value, TDSLOGIN * login);
 
@@ -507,7 +507,7 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 	char *s;
 	char p;
 	int i;
-	int insection = 0;
+	bool insection = false;
 	bool found = false;
 
 	tdsdump_log(TDS_DBG_INFO1, "Looking for section %s.\n", section);
@@ -573,10 +573,10 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 
 			if (!strcasecmp(section, &option[1])) {
 				tdsdump_log(TDS_DBG_INFO1, "Got a match.\n");
-				insection = 1;
+				insection = true;
 				found = true;
 			} else {
-				insection = 0;
+				insection = false;
 			}
 		} else if (insection) {
 			tds_conf_parse(option, value, param);
@@ -692,6 +692,7 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		s = tds_dstr_copy(&login->openssl_ciphers, value);
 	} else if (!strcmp(option, TDS_STR_ENABLE_TLS_V1)) {
 		login->enable_tls_v1 = tds_config_boolean(option, value, login);
+		login->enable_tls_v1_specified = 1;
 	} else {
 		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option '%s' ... ignoring.\n", option);
 	}
@@ -700,7 +701,7 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		login->valid_configuration = 0;
 }
 
-static int
+static bool
 tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 {
 	DSTR *res = &login->server_name;
@@ -804,34 +805,39 @@ tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 		connection->use_ntlmv2 = login->use_ntlmv2;
 	}
 
+	if (login->enable_tls_v1_specified) {
+		connection->enable_tls_v1_specified = login->enable_tls_v1_specified;
+		connection->enable_tls_v1 = login->enable_tls_v1;
+	}
+
 	if (res)
 		res = tds_dstr_dup(&connection->new_password, &login->new_password);
 
 	return res != NULL;
 }
 
-static int
+static bool
 tds_config_env_tdsdump(TDSLOGIN * login)
 {
 	char *s = getenv("TDSDUMP");
 	if (!s)
-		return 1;
+		return true;
 
 	if (!strlen(s)) {
 		char *path;
 		pid_t pid = getpid();
 		if (asprintf(&path, pid_logpath, (int) pid) < 0)
-			return 0;
+			return false;
 		if (!tds_dstr_set(&login->dump_file, path)) {
 			free(path);
-			return 0;
+			return false;
 		}
 	} else {
 		if (!tds_dstr_copy(&login->dump_file, s))
-			return 0;
+			return false;
 	}
 	tdsdump_log(TDS_DBG_INFO1, "Setting 'dump_file' to '%s' from $TDSDUMP.\n", tds_dstr_cstr(&login->dump_file));
-	return 1;
+	return true;
 }
 
 static void
@@ -860,7 +866,7 @@ tds_config_env_tdsver(TDSLOGIN * login)
 }
 
 /* TDSHOST env var, pkleef@openlinksw.com 01/21/02 */
-static int
+static bool
 tds_config_env_tdshost(TDSLOGIN * login)
 {
 	const char *tdshost;
@@ -868,20 +874,20 @@ tds_config_env_tdshost(TDSLOGIN * login)
 	struct addrinfo *addrs;
 
 	if (!(tdshost = getenv("TDSHOST")))
-		return 1;
+		return true;
 
 	if (TDS_FAILED(tds_lookup_host_set(tdshost, &login->ip_addrs))) {
 		tdsdump_log(TDS_DBG_WARN, "Name resolution failed for '%s' from $TDSHOST.\n", tdshost);
-		return 0;
+		return false;
 	}
 
 	if (!tds_dstr_copy(&login->server_host_name, tdshost))
-		return 0;
+		return false;
 	for (addrs = login->ip_addrs; addrs != NULL; addrs = addrs->ai_next) {
 		tdsdump_log(TDS_DBG_INFO1, "Setting IP Address to %s (%s) from $TDSHOST.\n",
 			    tds_addrinfo2str(addrs, tmp, sizeof(tmp)), tdshost);
 	}
-	return 1;
+	return true;
 }
 #define TDS_FIND(k,b,c) tds_find(k, b, TDS_VECTOR_SIZE(b), sizeof(b[0]), c)
 
@@ -992,6 +998,7 @@ tds_lookup_host(const char *servername)	/* (I) name of the server
 	memset(&hints, '\0', sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
 
 #ifdef AI_ADDRCONFIG
 	hints.ai_flags |= AI_ADDRCONFIG;
@@ -1037,7 +1044,7 @@ hexdigit(int c)
 {
 	if (c >= '0' && c <= '9')
 		return c - '0';
-	/* ascii optimization, 'A' -> 'a', 'a' -> 'a' */
+	/* ASCII optimization, 'A' -> 'a', 'a' -> 'a' */
 	c |= 0x20;
 	if (c >= 'a' && c <= 'f')
 		return c - 'a' + 10;
@@ -1058,9 +1065,9 @@ hex2num(char *hex)
  * \param dir name of base directory for interface file
  * \param file name of the interface file
  * \param host logical host to search for
- * \return 0 if not fount 1 if found
+ * \return false if not fount true if found
  */
-static int
+static bool
 search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const char *host)
 {
 	char *pathname;
@@ -1070,8 +1077,8 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 	char tmp_ver[sizeof(line)];
 	FILE *in;
 	char *field;
-	int found = 0;
-	int server_found = 0;
+	bool found = false;
+	bool server_found = false;
 	char *lasts;
 
 	line[0] = '\0';
@@ -1082,7 +1089,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 	tdsdump_log(TDS_DBG_INFO1, "Searching interfaces file %s/%s.\n", dir, file);
 	pathname = tds_new(char, strlen(dir) + strlen(file) + 10);
 	if (!pathname)
-		return 0;
+		return false;
 
 	/*
 	 * create the full pathname to the interface file
@@ -1106,7 +1113,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 	if ((in = fopen(pathname, "r")) == NULL) {
 		tdsdump_log(TDS_DBG_INFO1, "Couldn't open %s.\n", pathname);
 		free(pathname);
-		return 0;
+		return false;
 	}
 	tdsdump_log(TDS_DBG_INFO1, "Interfaces file %s opened.\n", pathname);
 
@@ -1117,10 +1124,10 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 		if (!TDS_ISSPACE(line[0])) {
 			field = strtok_r(line, "\n\t ", &lasts);
 			if (!strcmp(field, host)) {
-				found = 1;
+				found = true;
 				tdsdump_log(TDS_DBG_INFO1, "Found matching entry for host %s.\n", host);
 			} else
-				found = 0;
+				found = false;
 		} else if (found && TDS_ISSPACE(line[0])) {
 			field = strtok_r(line, "\n\t ", &lasts);
 			if (field != NULL && !strcmp(field, "query")) {
@@ -1145,7 +1152,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 					field = strtok_r(NULL, "\n\t ", &lasts);	/* port */
 					strcpy(tmp_port, field);
 				}	/* else */
-				server_found = 1;
+				server_found = true;
 			}	/* if */
 		}		/* else if */
 	}			/* while */
@@ -1161,7 +1168,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 		if (TDS_SUCCEED(tds_lookup_host_set(tmp_ip, &login->ip_addrs))) {
 			struct addrinfo *addrs;
 			if (!tds_dstr_copy(&login->server_host_name, tmp_ip))
-				return 0;
+				return false;
 			for (addrs = login->ip_addrs; addrs != NULL; addrs = addrs->ai_next) {
 				tdsdump_log(TDS_DBG_INFO1, "Resolved IP as '%s'.\n",
 					    tds_addrinfo2str(login->ip_addrs, line, sizeof(line)));
@@ -1183,10 +1190,10 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
  *
  * @note This function uses only the interfaces file and is deprecated.
  */
-static int
+static bool
 tds_read_interfaces(const char *server, TDSLOGIN * login)
 {
-	int found = 0;
+	bool found = false;
 
 	/* read $SYBASE/interfaces */
 
@@ -1275,7 +1282,7 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 
 		if (TDS_SUCCEED(tds_lookup_host_set(server, &login->ip_addrs)))
 			if (!tds_dstr_copy(&login->server_host_name, server))
-				return 0;
+				return false;
 
 		if (ip_port)
 			login->port = ip_port;
@@ -1287,9 +1294,9 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 /**
  * Check the server name to find port info first
  * Warning: connection-> & login-> are all modified when needed
- * \return 1 when found, else 0
+ * \return true when found, else false
  */
-static int
+static bool
 parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server)
 {
 	const char *pSep;
@@ -1315,17 +1322,24 @@ parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_
 		/* handle instance name */
 		pSep = strrchr(server, '\\');
 		if (!pSep || pSep == server)
-			return 0;
+			return false;
 
 		if (!tds_dstr_copy(&connection->instance_name, pSep + 1))
-			return 0;
+			return false;
 		connection->port = 0;
 	}
 
-	if (!update_server || !tds_dstr_copyn(&connection->server_name, server, pSep - server))
-		return 0;
+	if (!update_server)
+		return true;
 
-	return 1;
+	if (server[0] == '[' && pSep > server && pSep[-1] == ']') {
+		server++;
+		pSep--;
+	}
+	if (!tds_dstr_copyn(&connection->server_name, server, pSep - server))
+		return false;
+
+	return true;
 }
 
 /**
